local AGS = AwesomeGuildStore

local WrapFunction = AGS.internal.WrapFunction
local RegisterForEvent = AGS.internal.RegisterForEvent
local chat = AGS.internal.chat

local GuildHistoryHelper = ZO_InitializingObject:Subclass()
AGS.class.GuildHistoryHelper = GuildHistoryHelper

function GuildHistoryHelper:Initialize()
    self.state = {}
    if GetAPIVersion() >= 101041 then return end
    RegisterForEvent(EVENT_GUILD_HISTORY_CATEGORY_UPDATED, function(_, guildId, category)
        if category == GUILD_HISTORY_STORE then
            local guildState = self:GetOrInitializeGuildState(guildId)
            local oldest, newest, endIndex = self:GetMinMaxPurchaseEventTimes(guildId, guildState.highestIndex)
            guildState.hasMore = DoesGuildHistoryCategoryHaveMoreEvents(guildId, category)
            guildState.highestIndex = endIndex
            guildState.oldest = oldest
            guildState.newest = newest
        end
    end)
end

function GuildHistoryHelper:GetNumPurchaseEvents(guildId)
    if GetAPIVersion() >= 101041 then return 0 end
    return GetNumGuildEvents(guildId, GUILD_HISTORY_STORE)
end

function GuildHistoryHelper:GetPurchaseEvent(guildId, index)
    if GetAPIVersion() >= 101041 then return end
    local eventType, secsSinceEvent, sellerName, buyerName, itemCount, itemLink, sellPrice, tax = GetGuildEventInfo(guildId, GUILD_HISTORY_STORE, index)
    local subcategory = ComputeGuildHistoryEventSubcategory(eventType, GUILD_HISTORY_STORE)
    if subcategory == GUILD_HISTORY_STORE_PURCHASES then
        return eventType, secsSinceEvent, sellerName, buyerName, itemCount, itemLink, sellPrice, tax
    end
end

function GuildHistoryHelper:GetMinMaxPurchaseEventTimes(guildId, startIndex)
    startIndex = math.max(1, startIndex or 1)
    local endIndex = self:GetNumPurchaseEvents(guildId)
    local oldest, newest = math.huge, 0
    if GetAPIVersion() < 101041 then
        for index = startIndex, endIndex do
            local eventType, secsSinceEvent = self:GetPurchaseEvent(guildId, index)
            if eventType then
                local time = ZO_NormalizeSecondsSince(secsSinceEvent)
                oldest = math.min(oldest, time)
                newest = math.max(newest, time)
            end
        end
    end
    return oldest, newest, endIndex
end

function GuildHistoryHelper:GetOrInitializeGuildState(guildId)
    if GetAPIVersion() >= 101041 then return end
    if not self.state[guildId] then
        local oldest, newest, endIndex = self:GetMinMaxPurchaseEventTimes(guildId)
        self.state[guildId] = {
            hasMore = DoesGuildHistoryCategoryHaveMoreEvents(guildId, GUILD_HISTORY_STORE),
            highestIndex = endIndex,
            oldest = oldest,
            newest = newest
        }
    end
    return self.state[guildId]
end

function GuildHistoryHelper:RequestEvents(guildId)
    if GetAPIVersion() >= 101041 then return false end
    if RequestMoreGuildHistoryCategoryEvents(guildId, GUILD_HISTORY_STORE, true) then
        return true
    end
    return false
end

function GuildHistoryHelper:RequestData(normalizedTime)
    if GetAPIVersion() >= 101041 then return false end
    local hasRequestedData = false
    for i = 1, GetNumGuilds() do
        local guildId = GetGuildId(i)
        local guildState = self:GetOrInitializeGuildState(guildId)
        if not guildState.initialized or (normalizedTime < guildState.oldest and guildState.hasMore) then
            guildState.initialized = true
            if self:RequestEvents(guildId) then
                hasRequestedData = true
            end
        end
    end
    return hasRequestedData
end
